@@ -59,7 +59,7 @@ float get_distance(float* codebook, svm_node **sparseData,
 void get_bmu_coord(float* codebook, svm_node **sparseData,
                    unsigned int nSomY, unsigned int nSomX,
                    unsigned int nDimensions, int* coords, unsigned int n) {
-    float mindist = 9999.99;
+    float mindist = 0.0f;
     float dist = 0.0f;
 
     /// Check nSomX * nSomY nodes one by one and compute the distance
@@ -69,7 +69,7 @@ void get_bmu_coord(float* codebook, svm_node **sparseData,
         for (unsigned int som_x = 0; som_x < nSomX; som_x++) {
             dist = get_distance(codebook, sparseData, som_y, som_x, nSomX,
                                 nDimensions, n);
-            if (dist < mindist) {
+            if ((som_y == 0 && som_x == 0) || (dist < mindist)) {
                 mindist = dist;
                 coords[0] = som_x;
                 coords[1] = som_y;
@@ -84,9 +84,15 @@ void trainOneEpochSparseCPU(int itask, svm_node **sparseData, float *numerator,
                             unsigned int nDimensions, unsigned int nVectors,
                             unsigned int nVectorsPerRank, float radius,
                             float scale, string mapType, string gridType,
-                            bool compact_support, bool gaussian, int *globalBmus) {
+                            bool compact_support, bool gaussian,
+                            int *globalBmus, bool only_bmus) {
     int p1[2] = {0, 0};
-    int *bmus = new int[nVectorsPerRank * 2];
+    int *bmus;
+#ifdef HAVE_MPI
+    bmus = new int[nVectorsPerRank * 2];
+#else
+    bmus = globalBmus;
+#endif
 #ifdef _OPENMP
     #pragma omp parallel default(shared) private(p1)
 #endif
@@ -108,29 +114,50 @@ void trainOneEpochSparseCPU(int itask, svm_node **sparseData, float *numerator,
             }
         }
     }
-
+    if (only_bmus) {
+#ifdef HAVE_MPI
+        MPI_Gather(bmus, nVectorsPerRank * 2, MPI_INT, globalBmus, nVectorsPerRank * 2, MPI_INT, 0, MPI_COMM_WORLD);
+        delete [] bmus;
+#endif
+        return;
+    }
+#ifdef HAVE_MPI
     float *localNumerator = new float[nSomY * nSomX * nDimensions];
     float *localDenominator = new float[nSomY * nSomX];
 #ifdef _OPENMP
     #pragma omp parallel default(shared)
-#endif
+#endif // _OPENMP
     {
 #ifdef _OPENMP
         #pragma omp for
-#endif
+#endif // _OPENMP
 #ifdef _WIN32
         for (int som_y = 0; som_y < nSomY; som_y++) {
 #else
         for (unsigned int som_y = 0; som_y < nSomY; som_y++) {
-#endif
+#endif // _WIN32
             for (unsigned int som_x = 0; som_x < nSomX; som_x++) {
                 localDenominator[som_y * nSomX + som_x] = 0.0;
                 for (unsigned int d = 0; d < nDimensions; d++)
                     localNumerator[som_y * nSomX * nDimensions + som_x * nDimensions + d] = 0.0;
             }
         }
-
-        /// Accumulate denoms and numers
+    }
+#ifdef _OPENMP
+    #pragma omp parallel default(shared)
+#endif
+#else  // not HAVE_MPI
+    float *localNumerator;
+    float localDenominator;
+    // Accumulate denoms and numers
+#ifdef _OPENMP
+    #pragma omp parallel default(shared) private(localDenominator) private(localNumerator)
+#endif
+#endif // HAVE_MPI
+    {
+#ifndef HAVE_MPI
+        localNumerator = new float[nDimensions];
+#endif // HAVE_MPI
 #ifdef _OPENMP
         #pragma omp for
 #endif
@@ -139,8 +166,12 @@ void trainOneEpochSparseCPU(int itask, svm_node **sparseData, float *numerator,
 #else
         for (unsigned int som_y = 0; som_y < nSomY; som_y++) {
 #endif
-
             for (unsigned int som_x = 0; som_x < nSomX; som_x++) {
+#ifndef HAVE_MPI
+                localDenominator = 0;
+                for (unsigned int d = 0; d < nDimensions; d++)
+                    localNumerator[d] = 0.0;
+#endif
                 for (unsigned int n = 0; n < nVectorsPerRank; n++) {
                     if (itask * nVectorsPerRank + n < nVectors) {
                         float dist = 0.0f;
@@ -161,38 +192,49 @@ void trainOneEpochSparseCPU(int itask, svm_node **sparseData, float *numerator,
                             }
                         }
                         float neighbor_fuct = getWeight(dist, radius, scale, compact_support, gaussian);
+#ifdef HAVE_MPI
                         unsigned int j = 0;
                         while ( sparseData[n][j].index != -1 ) {
                             localNumerator[som_y * nSomX * nDimensions +
                                            som_x * nDimensions +
                                            sparseData[n][j].index] +=
                                                1.0f * neighbor_fuct * sparseData[n][j].value;
-                            j++;
+                            ++j;
                         }
                         localDenominator[som_y * nSomX + som_x] += neighbor_fuct;
+#else // In this case, we can update in place
+                        unsigned int j = 0;
+                        localDenominator += neighbor_fuct;
+                        while ( sparseData[n][j].index != -1 ) {
+                            localNumerator[sparseData[n][j].index] += 1.0f * neighbor_fuct * sparseData[n][j].value;
+                            ++j;
+                        }
+#endif                        
+                    }
+                } // Looping over data instances
+#ifndef HAVE_MPI // We update in-place
+                for (unsigned int d = 0; d < nDimensions; d++) {
+                    float newWeight = localNumerator[d] / localDenominator;
+                    if (newWeight > 0.0) {
+                        codebook[som_y * nSomX * nDimensions + som_x * nDimensions + d] = newWeight;
                     }
                 }
-            }
-        }
-    }
+#endif
+            } // Looping over som_x
+        } // Looping over som_y
+#ifndef HAVE_MPI
+    delete [] localNumerator;
+#endif
+    } // OPENMP
 #ifdef HAVE_MPI
     MPI_Reduce(localNumerator, numerator,
                nSomY * nSomX * nDimensions, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);
     MPI_Reduce(localDenominator, denominator,
                nSomY * nSomX, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);
     MPI_Gather(bmus, nVectorsPerRank * 2, MPI_INT, globalBmus, nVectorsPerRank * 2, MPI_INT, 0, MPI_COMM_WORLD);
-#else
-    for (unsigned int i = 0; i < nSomY * nSomX * nDimensions; ++i) {
-        numerator[i] = localNumerator[i];
-    }
-    for (unsigned int i = 0; i < nSomY * nSomX; ++i) {
-        denominator[i] = localDenominator[i];
-    }
-    for (unsigned int i = 0; i < 2 * nVectorsPerRank; ++i) {
-        globalBmus[i] = bmus[i];
-    }
-#endif
-    delete [] bmus;
-    delete [] localNumerator;
+
     delete [] localDenominator;
+    delete [] localNumerator;
+    delete [] bmus;
+#endif
 }
